<?php

namespace FiloBlu\ExtAmastyGeoipRedirect\Helper;

use FiloBlu\ExtAmastyGeoipRedirect\Model\Source\Logic;
use Magento\Store\Model\ScopeInterface;

class Data extends \Magento\Framework\App\Helper\AbstractHelper {

    protected $_countries;
    protected $_countryFactory;

    /**
     * @var \Amasty\Geoip\Model\Geolocation
     */
    protected $geolocation;

    /**
     * @var \Magento\Framework\HTTP\PhpEnvironment\RemoteAddress
     */
    protected $remoteAddress;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;
    protected $geoData;
protected $_multiDomainHelper;
    
    public function __construct(
    \Magento\Framework\App\Helper\Context $context, \Magento\Directory\Model\Config\Source\Country $countries, \Magento\Directory\Model\CountryFactory $countryFactory, \Amasty\Geoip\Model\Geolocation $geolocation, \Magento\Store\Model\StoreManagerInterface $storeManager, \FiloBlu\MultiDomain\Helper\Data $multiDomainHelper
    ) {
        $this->_countries = $countries;
        $this->_countryFactory = $countryFactory;
        $this->geolocation = $geolocation;
        $this->remoteAddress = $context->getRemoteAddress();
        $this->storeManager = $storeManager;
        $this->_multiDomainHelper = $multiDomainHelper;

        parent::__construct($context);
    }

    public function getGeoData() {
        if (!$this->geoData) {
            $currentStoreId = $this->storeManager->getStore()->getId();
            $scopeStores = ScopeInterface::SCOPE_STORES;

            $currentIp = $this->remoteAddress->getRemoteAddress();
            $fakeIp = $this->scopeConfig->getValue('amgeoipredirect/debug/fake_ip', $scopeStores, $currentStoreId);
            $fakeCountry = $this->scopeConfig->getValue('amgeoipredirect/debug/fake_country', $scopeStores, $currentStoreId);

            if ($fakeIp) {
                $currentIp = $fakeIp;
            }

            $location = $this->geolocation->locate($currentIp);
            $country = $location->getCountry();

            if ($fakeCountry) {
                $country = $fakeCountry;
            }

            $this->geoData = new \Magento\Framework\DataObject();

            if ($country) {
                $this->geoData->setCountryCode($country);
                $this->geoData->setCountryName($this->getCountryName($country, "en_US"));

                $allStores = $this->storeManager->getStores();

                $currentGroup = $this->_multiDomainHelper->getCurrentGroup();

                foreach ($allStores as $store) {

                    if (!$store->isActive() || ($currentGroup && $currentGroup != $this->_multiDomainHelper->getCurrentGroup($store->getWebsite()->getCode()))) {
                        continue;
                    }

                    $countries = $this->scopeConfig->getValue('amgeoipredirect/country_store/affected_countries', $scopeStores, $store->getId());

                    if ($country && $countries && strpos($countries, $country) !== false) {
                        $this->geoData->setStore($store);
                        break;
                    }
                }
            }

            return $this->geoData;
        } else {
            return $this->geoData;
        }
    }

    public function getAllCountries() {
        return $this->_countryFactory->create()->getCollection()->getData();
    }

    public function getCountryName($iso = null, $locale = null) {

        $country = $this->_countryFactory->create()->loadByCode($iso);

        return $country->getName($locale);
    }

    public function setCountryCookie($country) {

        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $objectManager->get('FiloBlu\ExtAmastyGeoipRedirect\Cookie\Country')
                ->set($country);
    }

    public function getCountryCookie() {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        return $objectManager->get('FiloBlu\ExtAmastyGeoipRedirect\Cookie\Country')
                        ->get();
    }

    public function getCountryCookieName() {
        return \FiloBlu\ExtAmastyGeoipRedirect\Cookie\Country::COOKIE_NAME;
    }

    public function setLocalizedCountryCookie($country) {

        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $objectManager->get('FiloBlu\ExtAmastyGeoipRedirect\Cookie\LocalizedCountry')
                ->set($country);
    }

    public function getLocalizedCountryCookie() {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        return $objectManager->get('FiloBlu\ExtAmastyGeoipRedirect\Cookie\LocalizedCountry')
                        ->get();
    }

    public function getLocalizedCountryCookieName() {
        return \FiloBlu\ExtAmastyGeoipRedirect\Cookie\LocalizedCountry::COOKIE_NAME;
    }

    public function setStoreCookie($storeId) {

        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $objectManager->get('FiloBlu\ExtAmastyGeoipRedirect\Cookie\Store')
                ->set($storeId);
    }

    public function getStoreCookie() {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        return $objectManager->get('FiloBlu\ExtAmastyGeoipRedirect\Cookie\Store')
                        ->get();
    }

    public function getMisfitsCookieName() {
        return \FiloBlu\ExtAmastyGeoipRedirect\Cookie\Misfits::COOKIE_NAME;
    }

    public function setMisfitsCookie() {

        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $objectManager->get('FiloBlu\ExtAmastyGeoipRedirect\Cookie\Misfits')
                ->set(1);
    }

    public function getMisfitsCookie() {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        return $objectManager->get('FiloBlu\ExtAmastyGeoipRedirect\Cookie\Misfits')
                        ->get();
    }

    public function getStoreCookieName() {
        return \FiloBlu\ExtAmastyGeoipRedirect\Cookie\Store::COOKIE_NAME;
    }

    public function setCurrencyCookie() {

        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $objectManager->get('FiloBlu\ExtAmastyGeoipRedirect\Cookie\Currency')
                ->set(1);
    }

    public function getCurrencyCookie() {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        return $objectManager->get('FiloBlu\ExtAmastyGeoipRedirect\Cookie\Currency')
                        ->get();
    }

    public function getCurrencyCookieName() {
        return \FiloBlu\ExtAmastyGeoipRedirect\Cookie\Currency::COOKIE_NAME;
    }

    public function setUrlCookie() {

        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $objectManager->get('FiloBlu\ExtAmastyGeoipRedirect\Cookie\Url')
                ->set(1);
    }

    public function getUrlCookie() {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        return $objectManager->get('FiloBlu\ExtAmastyGeoipRedirect\Cookie\Url')
                        ->get();
    }

    public function getUrlCookieName() {
        return \FiloBlu\ExtAmastyGeoipRedirect\Cookie\Url::COOKIE_NAME;
    }

    public function getCanLogCookie() {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        return $objectManager->get('FiloBlu\ExtAmastyGeoipRedirect\Cookie\CanLog')
                        ->get();
    }

    public function getCurrentCountryCode() {
        if ($this->getCountryCookie()) {

            $countries = $this->scopeConfig->getValue('amgeoipredirect/country_store/affected_countries', \Magento\Store\Model\ScopeInterface::SCOPE_STORES);
            $countries = explode(",", $countries);
            $countryFound = false;

            foreach ($countries as $country) {
                if ($country == $this->getCountryCookie()) {
                    return $country;
                }
            }

            return $this->scopeConfig->getValue('general/country/default', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        } else {
            return $this->scopeConfig->getValue('general/country/default', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        }



//        return ($this->_helper->getCountryCookie() ? $this->_helper->getCountryCookie() : $this->_scopeConfig->getValue('general/country/default', \Magento\Store\Model\ScopeInterface::SCOPE_STORE));
    }

    public function getCurrentCountryName($locale = null) {
        return $this->getCountryName($this->getCurrentCountryCode(), $locale);
    }
    
    
    public function applyLogic($request, $logEnabled, $currentUrl, $logger) {

        $applyLogic = $this->scopeConfig->getValue('amgeoipredirect/restriction/apply_logic');

        $redirectAllowed = false;
        
        if ($this->geolocationRequest($request)) {
            return true;
        }

        switch ($applyLogic) {
            case Logic::ALL_URLS :
                return true;
                break;
            case Logic::SPECIFIED_URLS :
                $acceptedUrls = explode(PHP_EOL, $this->scopeConfig->getValue('amgeoipredirect/restriction/accepted_urls'));
                foreach ($acceptedUrls as $url) {
                    $url = trim($url);
                    if ($url && $currentUrl && strpos($currentUrl, $url) !== false) {
                        return true;
                    }
                }
                break;
            case Logic::EXCEPT_URLS :
                $exceptedUrls = explode(PHP_EOL, $this->scopeConfig->getValue('amgeoipredirect/restriction/excepted_urls'));
                foreach ($exceptedUrls as $url) {
                    $url = trim($url);
                    if ($url && $currentUrl && strpos($currentUrl, $url) !== false) {
                        return false;
                    } else {
                        $redirectAllowed = true;
                    }
                }
                break;
            case Logic::HOMEPAGE_ONLY :
                $routeName = $request->getRouteName();
                $action = $request->getActionName();
                if ($routeName == 'cms' && $action == 'index') {
                    $redirectAllowed = true;
                }
                break;
            case Logic::SPECIFIED_PATH :
                $acceptedPath = explode(PHP_EOL, $this->scopeConfig->getValue('amgeoipredirect/restriction/accepted_path'));
                $currentPath = parse_url($currentUrl, PHP_URL_PATH);

                if ($logEnabled) {
                    $logger->info("Current path " . $currentPath);
                }

                foreach ($acceptedPath as $path) {
                    $path = trim($path);
                    if ($currentPath && $path && $path == $currentPath) {
                        return true;
                    }
                }
                break;
        }
        
        return $redirectAllowed;
    }
    
    public function geolocationRequest($request) {
        return (null !== $request->getParam("geolocation") && $request->getParam("geolocation") == 1);
    }
    
    public function notGeolocalizedRequest($request) {
        return (null !== $request->getParam("notgeolocalized") && $request->getParam("notgeolocalized") == 1);
    }

}
